<p align="center">
    <a href="https://sylius.com" target="_blank">
        <img src="https://demo.sylius.com/assets/shop/img/logo.png" />
    </a>
</p>

<h1 align="center">Plugin Skeleton</h1>

<p align="center">Skeleton for starting Sylius plugins.</p>

## Documentation

For a comprehensive guide on Sylius Plugins development please go to Sylius documentation,
there you will find the <a href="https://docs.sylius.com/en/latest/plugin-development-guide/index.html">Plugin Development Guide</a>, that is full of examples.

## Quickstart Installation

```bash
$ make start
$ make db-create
$ make db-update
$ make fixtures
$ make build
```

## Installation

1. [Install Sylius](https://docs.sylius.com/en/latest/book/installation/installation.html)
2. Install **Sylius Dedi Customer Referral**: `composer require dedi/sylius-customer-referral-plugin`
3. Register the bundle:

    ```php
    <?php

    // config/bundles.php

    return [
        // ...
        Dedi\SyliusCustomerReferralPlugin\DediSyliusCustomerReferralPlugin::class => ['all' => true],
    ];

4. Import the configuration:

    ```yaml
    # config/packages/_sylius.yaml
    imports:
        - { resource: "@DediSyliusCustomerReferralPlugin/Resources/config/config.yaml" }
     ```

5. Import the routes:

    ```yaml
    # config/routes/routes.yaml
    dedi_customer_referral:
        resource: "@DediSyliusCustomerReferralPlugin/Resources/config/shop_routes.yaml"
        prefix: /{_locale}
        requirements:
            _locale: ^[a-z]{2}(?:_[A-Z]{2})?$
    ```

6. Update the ShopUserEntity

    ```php
    <?php

    declare(strict_types=1);

    namespace App\Entity;

    use Dedi\SyliusCustomerReferralPlugin\Entity\InviterTrait;
    use Dedi\SyliusCustomerReferralPlugin\Entity\InviteeTrait;
    use Dedi\SyliusCustomerReferralPlugin\Entity\InviteeInterface;
    use Dedi\SyliusCustomerReferralPlugin\Entity\InviterInterface;
    use Doctrine\ORM\Mapping as ORM;
    use Sylius\Component\Core\Model\ShopUser as BaseShopUser;
    use Sylius\Component\Core\Model\ShopUserInterface;

    /**
     * @ORM\Entity()
     * @ORM\Table(name="sylius_shop_user")
     */
    class ShopUser extends BaseShopUser implements ShopUserInterface, InviterInterface, InviteeInterface
    {
        use InviterTrait {
            InviterTrait::__construct as private __inviterConstruct;
        }

        use InviteeTrait;

        public function __construct()
        {
            parent::__construct();

            $this->__inviterConstruct();
        }

        // ...

    }
    ```

7. Override the templates:

       mkdir -p templates/bundles/DediSyliusCustomerReferralPlugin/
       cp -R vendor/dedi/sylius-customer-referral-plugin/src/Resources/views/* templates/bundles/DediSyliusCustomerReferralPlugin/

8. Create and execute database migrations:

       bin/console doctrine:migrations:diff
       bin/console doctrine:migrations:migrate

9. Create listeners (GenericEvent are dispatched when a referral is accepted):

    | event | $subject type |
    |-------|---------------|
    | `dedi_sylius_customer_referral_plugin.referral_accepted` | Referral | 
    | `dedi_sylius_customer_referral_plugin.inviter_has_new_registered_invitee` | Customer | 
    | `dedi_sylius_customer_referral_plugin.invitee_accept_invite` | Customer |
    | `dedi_sylius_customer_referral_plugin.inviter_has_invitee_order_complete` | Customer |

## Usage

### Running plugin tests

  - PHPUnit

    ```bash
    $ vendor/bin/phpunit
    ```

  - PHPSpec

    ```bash
    $ vendor/bin/phpspec run
    ```

  - Behat (non-JS scenarios)

    ```bash
    $ vendor/bin/behat --tags="~@javascript"
    ```

  - Behat (JS scenarios)
 
    1. Download [Chromedriver](https://sites.google.com/a/chromium.org/chromedriver/)
    
    2. Download [Selenium Standalone Server](https://www.seleniumhq.org/download/).
    
    2. Run Selenium server with previously downloaded Chromedriver:
    
        ```bash
        $ java -Dwebdriver.chrome.driver=chromedriver -jar selenium-server-standalone.jar
        ```
        
    3. Run test application's webserver on `localhost:8080`:
    
        ```bash
        $ (cd tests/Application && bin/console server:run localhost:8080 -d public -e test)
        ```
    
    4. Run Behat:
    
        ```bash
        $ vendor/bin/behat --tags="@javascript"
        ```

### Opening Sylius with your plugin

After installation and with docker containers running, go to http://0.0.0.0:9000/
