<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Application\Form\Shop\Extension;

use Dedi\SyliusCustomerReferralPlugin\Infrastructure\Context\RegisterReferralContext;
use Sylius\Bundle\CoreBundle\Form\Type\Customer\CustomerRegistrationType;
use Symfony\Component\Form\AbstractTypeExtension;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\FormBuilderInterface;

class CustomerRegistrationTypeExtension extends AbstractTypeExtension
{
    /** @var RegisterReferralContext */
    private $registerReferralContext;

    public function __construct(
        RegisterReferralContext $registerReferralContext
    ) {
        $this->registerReferralContext = $registerReferralContext;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        if (!$referral = $this->registerReferralContext->getReferral()) {
            return;
        }

        $builder
            ->remove('email')
            ->add('email', EmailType::class, [
                'label' => 'sylius.form.customer.email',
                'attr' => [
                    'readonly' => 'readonly',
                    'class' => 'ui disabled input',
                ],
                'empty_data' => $referral->getInviteeEmail(),
                'data' => $referral->getInviteeEmail(),
            ])
            ->add('code', HiddenType::class, [
                'data' => $referral->getCode(),
                'mapped' => false,
            ])
        ;
    }

    public static function getExtendedTypes(): array
    {
        return [CustomerRegistrationType::class];
    }
}
