<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Domain\Referral\UseCase\Invite;

use Dedi\SyliusCustomerReferralPlugin\Domain\Common\Email\Email;
use Dedi\SyliusCustomerReferralPlugin\Domain\Common\Email\EmailSenderInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\InviteeAlreadyInvitedException;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Factory\ReferralFactoryInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Repository\ReferralRepositoryInterface;

class Invite
{
    /** @var ReferralFactoryInterface */
    private $referralFactory;

    /** @var ReferralRepositoryInterface */
    private $referralRepository;

    /** @var EmailSenderInterface */
    private $emailSender;

    public function __construct(
        ReferralFactoryInterface $referralFactory,
        ReferralRepositoryInterface $referralRepository,
        EmailSenderInterface $emailSender
    ) {
        $this->referralFactory = $referralFactory;
        $this->referralRepository = $referralRepository;
        $this->emailSender = $emailSender;
    }

    /**
     * @throws InviteeAlreadyInvitedException
     */
    public function __invoke(
        InviteInput $input,
        InvitePresenterInterface $presenter
    ): void {
        $referralModel = $this->referralFactory->createReferral($input->toReferralDTO());
        $referralModel = $this->referralRepository->save($referralModel);

        $this->emailSender->send(new Email('invite', [$input->getEmail()], [
            'referral' => $referralModel,
        ]));

        $presenter->present(new InviteOutput($referralModel));
    }
}
